/*
   NAME:     p01
   AUTHOR:   Ole Gunnar Westgaard <olew@maud.ifi.uio.no>
   DATE:     Aug  9 1995
             First example program of Otools
*/

#include "otools.h"  // class declarations and defines
#include "colors.h"  // color defines
#include "macros.h"  // Macros to draw in your window
#include "protos.h"  // Declaration of procedures and variables in Otools

int main(int argc,char *argv[])
{
  // Start the X window
  o_Init("Mitt vindu","Mitt ikon","7x13bold",50,50,160,120,320,240,640,480,
	 C_GRAY,argc,argv);

  // Start message filtering loop
  o_EnterLoop();

  // Stop the application
  o_Quit();
}

