/*
   File:   colors.h
   Author: Ole Gunnar Westgaard 
           root@ogware.oslo.no
           Copyright (c) May 1995.  All Rights Reserved.
*/

#ifndef COLORS
#define COLORS

/*
  border,shadow,background,highlight,foreground,dark,light
 */

#define RGB_LAKE   "#000","#239","#45c","#65f","#000","#444","#000"
#define RGB_PINK   "#000","#755","#a77","#daa","#000","#444","#000"
#define RGB_GRAY   "#000","#666","#888","#ccc","#ff0","#008","#f00"
#define RGB_RED    "#000","#600","#a00","#f00","#fff","#000","#ff0"
#define RGB_BLUE   "#000","#006","#00a","#00f","#fff","#003","#0ff"
#define RGB_GREEN  "#000","#060","#0a0","#0d0","#0ff","#aa0","#ff0"
#define C_PEACH    "black","peachpuff4","peachpuff3","peachpuff1",\
                   "royalblue4","black","yellow"

#define C_GREEN    "black","green4","green3","green1",\
                   "cyan","black","yellow"
#define C_BLUE     "Black","Blue4","Blue3","Blue2",\
                   "White","Gray20","White"
#define C_GRAY     "black","gray40","gray60","gray80",\
                   "black","Blue4","red2"
#define C_DGRAY    "black","gray20","gray40","gray60",\
                   "black","black","yellow"
#define C_RED      "Black","red4","red3","red1",\
                   "White","Black","Yellow"
#endif /* COLORS */





